:- discontiguous holdsAt/2, happens/2.
:- multifile holdsAt/2, happens/2.
:- dynamic holdsAt/2, happens/2, ttime/1.

:-['util.prolog'].

clearKB:-
    abolish(happens/2),
    abolish(holdsAt/2),
    abolish(ttime/1).

openData(FileNames):-
    clearKB,
    multifile(holdsAt/2),
    multifile(happens/2),
    dynamic(happens/2),
    dynamic(ttime/1),
    write('Loading source dataset files: '),nl,
    compile(FileNames),
    assertAllTimePoints.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



ids(IDs):-
  findall(ID, holdsAt(appearance(ID)=_,_),IDs1),
  sort(IDs1, IDs).

pairz(L, Result):- pairz(L,L,Result).

pairz([], _, [] ).

pairz([H|T], List, [R| Rest] ):-
%   cartesian([H], List, R),
   cartesian2([H], List, R),
  pairz(T, T, Rest).

% 
% Extraction predicates:
% 
extractHLE_Pair(ID1, ID2, Filename):-
  not(ID1 = ID2 ),
  open(Filename, write, Stream),
  findall(Result, (happens(Event,Time), Event =.. [_ | [_, [ID1, ID2]]], conv_happensHLE(Event,Time,Result)), L ),
  findall(Result2, (happens(Event2,Time2), Event2 =.. [_ | [_, [ID2, ID1]]], conv_happensHLE(Event2,Time2,Result2)), L2 ),
  writeList(L,Stream),
  writeList(L2,Stream),
  close(Stream),!.

extractHLE_Pair(ID, ID, Filename):-
  open(Filename, write, Stream),
  findall(Result, (happens(Event,Time), Event =.. [_ | [_, [ID, ID]]], conv_happensHLE(Event,Time,Result)), L ),
  writeList(L,Stream),
  close(Stream),!.


extractLLE_Pair(ID1, ID2, Filename):-
  not(ID1 = ID2),
  open(Filename, write, Stream),
  allLLEof(ID1, L1),
  writeList(L1,Stream),
  allLLEof(ID2, L2),
  writeList(L2,Stream),
  close(Stream),!.

extractLLE_Pair(ID, ID, Filename):-
  open(Filename, write, Stream),
  allLLEof(ID, L1),
  writeList(L1,Stream),
  close(Stream),!.


allLLEof(ID, LLEs):-
  findall(REnt, (holdsAt(appearance(ID)=appear, TEnt), conv_enterLLE(ID,TEnt,REnt)), ResultsEnter),
  findall(LLEResult, (happens(Event, Time), Event =.. [_ | [ID]], conv_happensLLE(Event,Time,LLEResult)), ResultsEvents),
  findall(RExt, (holdsAt(appearance(ID)=disappear, TExt), conv_exitLLE(ID,TExt,RExt)), ResultsExit),
  append(ResultsEnter, ResultsEvents, TMP),
  append(TMP,ResultsExit, LLEs),!.


extractClose_Pair(ID1, ID2, Filename):-
  not(ID1 = ID2),
  open(Filename, write, Stream),
  findall(R1 , (threshold_value(T1),closeTo(ID1,ID2,T1,R1)), Result1),
  flatten(Result1, ResultA),
  findall(R2 , (threshold_value(T2),closeTo(ID2,ID1,T2,R2)), Result2),
  flatten(Result2, ResultB),
  writeList(ResultA,Stream),
  writeList(ResultB,Stream),
  close(Stream),!.

extractClose_Pair(ID, ID, Filename):-
  open(Filename, write, Stream),
  findall(R1 , (threshold_value(T1),closeTo(ID,ID,T1,R1)), Result1),
  flatten(Result1, ResultA),
  writeList(ResultA,Stream),
  close(Stream),!.

  

extractOM_Pair(ID1, ID2, Filename):-
  not(ID1 = ID2),
  allOrientationMove([[ID1, ID2], [ID2,ID1]], Results),
  open(Filename, write, Stream),
  wrtOM(Stream,Results),
  close(Stream),!.

extractOM_Pair(ID, ID, Filename):-
  allOrientationMove([[ID, ID]], Results),
  open(Filename, write, Stream),
  wrtOM(Stream,Results),
  close(Stream),!.


writeIDs(ID1, ID2, Filename):-
  open(Filename, write, Stream),
  sort([ID1, ID2], Result),
  writeList(Result,Stream),
  close(Stream),!.

extractEvents(Filename):-
  open(Filename, write, Stream),
  findall(E, eventName(E), R1),
  sort(R1, R),
  writeList(['enter','exit'|R], Stream),
  close(Stream),!.
  
ids:-
  findall(ID, holdsAt(appearance(ID)=_,_),IDs1),
  sort(IDs1, IDs),
  writeList(IDs),!.


events:-
  findall(E, eventName(E), R1),
  sort(R1, R),
  write(R),!.

eventName(EventName):-
  happens(E,_T),
  E =.. [EventName|Arg], 
  Arg \= [_GRP_ID, _ListIds].


extractSeq(Filename):-
  open(Filename, write, Stream),
  getTimeBounds(Min, Max),
  wrtStartTime(Min, Stream),
  gen_sequence(Min, Max, Seq),
  writeList(Seq, Stream),
  close(Stream),!.

gen_sequence(T, T, []).

gen_sequence(StartTime,EndTime, [Result|T]) :-
  StartTime < EndTime,
  Current is StartTime / 40,
  Next is Current + 1,
  format(string(Result), 'Next(~a, ~a)', [Next,Current]),
  NextTime is StartTime + 40,
  gen_sequence(NextTime, EndTime, T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
 conv_closeLLE(ID1,ID2,Threshold,Time, Result) :-
   upcase_atom(ID1, NID1),
   upcase_atom(ID2, NID2),
   Frame is Time / 40,
   format(string(Result), 'Close(~a,~a,~a,~a)', [NID1,NID2,Threshold, Frame]).

conv_closeLLE2(ID1,ID2,Threshold,Time, Result1, Result2) :-
  upcase_atom(ID1, NID1),
  upcase_atom(ID2, NID2),
  Frame is Time / 40,
  format(string(Result1), 'Close(~a,~a,~a,~a)', [NID1,NID2,Threshold, Frame]),
  format(string(Result2), 'Close(~a,~a,~a,~a)', [NID2,NID1,Threshold, Frame]).

conv_happensLLE(Event,Time,Result):-
  Event=.. [EventName|Arg], 
  Arg \= [_GRP_ID, _ListIds],
  Arg = [ID],
  upcase_atom(ID, NID),
  uppercaseFirstChar(EventName, NewEventName),
  Frame is Time / 40,
  format(string(Result), 'Happens(~a_~a, ~a)', [NewEventName, NID, Frame]).

conv_enterLLE(ID,T,R):-
  upcase_atom(ID, NID),
  Frame is T/40,
  format(string(R), "Happens(Enter_~a, ~a)", [NID,Frame]).

conv_exitLLE(ID,T,R):-
  upcase_atom(ID, NID),
  Frame is (T+40)/40,
  format(string(R), "Happens(Exit_~a, ~a)", [NID,Frame]).

conv_happensHLE(Event,Time,Result):-
  Event=.. [EventName|Arg], 
  Arg = [_GRP_ID, [ID1, ID2]],
  upcase_atom(ID1, NID1),
  upcase_atom(ID2, NID2),
  uppercaseFirstChar(EventName, NewEventName),
  Frame is Time / 40,
  format(string(Result), 'HoldsAt(~a_~a_~a, ~a)', [NewEventName, NID1,NID2, Frame]).

% ===== THERSHOLD DISTANCES

browseDist( 24 ).
fightDist( 24 ).
moveDist( 34 ).
meetDist( 34 ).
interactDist( 25 ).
leaveDist( 30 ).


threshold_value(24).
threshold_value(25).
threshold_value(34).
threshold_value(30).


% ===== COORDINATES OF PLACES OF INTEREST

% place(shop1).
% place(shop2).
% place(shop3).

% holdsAt( coord( shop1 )=(86, 205), _ ).
% holdsAt( coord( shop2 )=(101, 260), _ ).
% holdsAt( coord( shop3 )=(181, 51), _ ).
	
% ===== STATE STATICALLY DETERMINED FLUENTS

sdFluent( distance(_, _) ).

sdFluent( closeImmobile(_, _) ).
sdFluent( close(_, _, _) ).

sdFluent( coord(_) ).
sdFluent( orientation(_) ).
sdFluent( appearance(_) ).



closeTo(ID1,ID2,Threshold,Result):-
  findall(R, (ttime(T), close(ID1,ID2,Threshold,T), conv_closeLLE(ID1,ID2,Threshold,T, R)), Result).

close(Person1, Person2, Threshold, T):-
  distance(Person1, Person2, Dist, T),
  Dist =< Threshold.

distance(Person1, Person2, Ypot, T):-
	holdsAt( coord(Person1) = (X1, Y1), T ),
	holdsAt( coord(Person2) = (X2, Y2), T ),
	\+ Person1 = Person2,
	XDiff is abs(X1-X2),
	YDiff is abs(Y1-Y2),
	SideA is XDiff*XDiff,
	SideB is YDiff*YDiff,
	Temp is SideA + SideB,
	Ypot is sqrt(Temp).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Orientation for move together
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


orientationMove(Person, Person2, T):-
  moveDist(Dist),
  close(Person, Person2, Dist, T),
  holdsAt( orientation(Person)=O, T ),
  holdsAt( orientation(Person2)=O2, T ),
  Diff is abs(O - O2), 
  Diff < 45.
  
allGrpActionIDs(Result):-
    findall(ID, holdsAt( appearance( ID )=_,  _ ), L1),
    sort(L1, L2),
    cartesian2(L2, L2, Result),!.

allOrientationMove([], []).

allOrientationMove([[P,P2]| TailGrp], [Result| TailResult]):-
  findall([P,P2,TimePoints], (allOrientationMoveFor(P,P2,TimePoints), not(TimePoints = [])), Result),
  not(Result = []),!,
  allOrientationMove(TailGrp, TailResult).

allOrientationMove([[P,P2]| TailGrp], TailResult):-
  findall([P,P2,TimePoints], (allOrientationMoveFor(P,P2,TimePoints), not(TimePoints = [])), Result),
  Result = [],!,
  allOrientationMove(TailGrp, TailResult).


allOrientationMoveFor(P,P2, Result):-
  findall(T, (ttime(T), orientationMove(P,P2,T)), Result),!.

allOrientationMove(Results):-
  allGrpActionIDs(Grps),
  allOrientationMove(Grps, Results).

wrtOM(Stream, [ [[ID1,ID2, [HeadTimePoint| TailTimePoints]]] | Tail]):-
  Frame is  HeadTimePoint /40,
  upcase_atom(ID1, NID1),
  upcase_atom(ID2, NID2),
  format(string(Result), 'OrientationMove(~a, ~a, ~a)', [NID1, NID2, Frame]),
  write(Stream,Result),nl(Stream),
  wrtOM(Stream,[ [[ID1,ID2, TailTimePoints]] | Tail]).
  
wrtOM(Stream,[[[_ID1,_ID2, []]] | Tail]):-
  wrtOM(Stream, Tail).

wrtOM(_, [[[_ID1,_ID2, []]]]).
wrtOM(_, []).

uppercaseFirstChar(Src, Result):-
   string_to_list(Src, [H|T]),
   string_to_list(Char, [H]),
   upcase_atom(Char, UpperChar),
   string_to_list(UpperChar, [Upper]),
   string_to_list(Result, [Upper|T]).	



wrtStartTime(Time, Stream):-
  Frame is Time / 40,
  format(string(Result), 'StartTime(~a)', [Frame]),
  write(Stream, Result),
  nl(Stream).
