:- dynamic ttime/1, tless/2, tlessEqual/2.


mergeStrings(String1,String2, Result):-
  string_to_list(String1, L1), 
  string_to_list(String2, L2), 
  append(L1,L2,L), 
  string_to_list(Result,L).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

generate_time(T, T). 
generate_time(TStart, TEnd):- 
	TStart=<TEnd, 
	assertz(ttime(TStart)), 
	NewStart is TStart + 1,
	generate_time(NewStart, TEnd).
	
generate_time(TStart, TEnd, _TimeStep):- 
	TStart >= TEnd,
	assertz(ttime(TEnd)).

generate_time(TStart, TEnd, TimeStep):- 
	TStart<TEnd, 
	assertz(ttime(TStart)), 
	NewStart is TStart + TimeStep,
	generate_time(NewStart, TEnd, TimeStep).


tless(A,B):- 
	ttime(A),ttime(B),
	A<B.

tgreatThan(A,B):-
	ttime(A),ttime(B),
	A>B.
	
tequal(A,B):-
	ttime(A), ttime(B),
	A == B.
	
tlessEqual(A,B):-
	ttime(A),ttime(B),
	A=<B.

tgreatEqual(A, B):-
	ttime(A),ttime(B),
	A>=B.


getAllTimePoints(SL):-
	findall(T, (happens(_,T), number(T), \+ ttime(T)), L),
	sort(L, SL).

assertAllTimePoints:-
	getAllTimePoints(SL),
	assertTimePoint(SL).
	
assertTimePoint([H|Tail]):-
	assert(ttime(H)),
	assertTimePoint(Tail).

assertTimePoint([]).


getAllTimePoints2(SL):-
	findall(T, ttime(T), L),
	sort(L, SL).

getTimeBounds(MinTime, MaxTime):-
	getAllTimePoints2(ListTime),
	min_list(ListTime, MinTime),
	max_list(ListTime, MaxTime).

getMaxTime(Max):-
	getAllTimePoints2(ListTime),
	max_list(ListTime, Max).

getMinTime(Min):-
	getAllTimePoints2(ListTime),
	min_list(ListTime, Min).


	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% KB utils
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

assertList([]).
assertList([Head| Tail]):-
	assertz(Head),
	assertList(Tail).
	
assertOnce([]).
assertOnce([Head| Tail]):-
    not(call(Head)),
	assertz(Head),
	assertOnce(Tail).
	
assertOnce([Head| Tail]):-
    call(Head),
	assertOnce(Tail).



assertOnce(A):-
    not(call(A)),
	assertz(A).

assertOnce(A):-
    call(A), fail.

	

retractList([]).
retractList([Head| Tail]):-
	retract(Head),
	retractList(Tail).
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% I/O Utils
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
writeListNarrative(L):-
	writeListNarrative(L,'user_output').

writeListNarrative([H|[]], Stream):-
	write(Stream, H), write(Stream, '.'), nl(Stream),
	close(Stream),!.

writeListNarrative([H|T],Stream):-
	write(Stream, H), write(Stream, ','), nl(Stream),
	writeListNarrative(T, Stream).
	
writeList(L):-
	writeList(L, 'user_output').

% writeList([], Stream):-
% 	close(Stream),!.

writeList([],_):- !.

writeList([H|T], Stream):-
	write(Stream, H), nl(Stream),
	writeList(T, Stream).
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


act_cmp(Delta, H1, H2):-
    H1 =.. [Name1 | [_, _, Time1] ],
    H2 =.. [Name2 | [_, _, Time2] ],
    (Name1 = 'initiates'; Name1='terminates'),
    (Name2 = 'initiates'; Name2='terminates'),
    time_cmp(Delta, Time1, Time2).
    
%time_cmp('=', T, T).    
time_cmp('<', T1, T2):- T1 =< T2.
time_cmp('>', T1, T2):- T1>T2.

% cartesian product
cartesian([], _L, []).
cartesian([A|N], L, M) :- 
             pair(A,L,M1), 
             cartesian(N, L, M2),
             append(M1, M2, M).
pair(_A, [], []).
pair(A, [B|L], [[A,B]|N] ) :- pair(A, L, N).

    
% cartesian product, but no pairs with the same elements allowed
cartesian2([], _L, []).
cartesian2([A|N], L, M) :- 
             pair2(A,L,M1), 
             cartesian2(N, L, M2),
             append(M1, M2, M).
pair2(_A, [], []).
pair2(A, [B|L], [[A,B]|N] ) :- pair2(A, L, N), A\=B.
pair2(A, [A|L], N ):- pair2(A, L, N).


% Creates a list containing all integers within a given range.
% range(I,K,L) :- I <= K, and L is the list containing all 
%    consecutive integers from I to K.
%    (integer,integer,list) (+,+,?)

range(I,I,[I]).

range(I,K,[I|L]) :- 
	I < K, 
	I1 is I + 1, 
	range(I1,K,L).

getCurrDateTime(DateTime):-
    get_time(Timestamp), 
    format_time(atom(DateTime), '%F_%T', Timestamp).
    
    
sameElements([], _).
sameElements([H|Tail], List):-
    lists:member(H, List),
    sameElements(Tail, List).
    
quick_sort(List,Sorted):-q_sort(List,[],Sorted).
q_sort([],Acc,Acc).
q_sort([H|T],Acc,Sorted):-
	pivoting(H,T,L1,L2),
	q_sort(L1,Acc,Sorted1),q_sort(L2,[H|Sorted1],Sorted).

pivoting(_H,[],[],[]).

pivoting(H,[X|T],[X|L],G):- 
	X=<H, 
	pivoting(H,T,L,G).
	
pivoting(H,[X|T],L,[X|G]):- 
	X > H, 
	pivoting(H,T,L,G).

% rangeFreq(7,8, [[7.2, 2], [7.9, 3], [8.1, 7], [7.6, 2], [9.2, 8], [7, 3]], R).
rangeFreq(_, _, [], 0). 
rangeFreq(Rmin, Rmax, [[Value, Counter]| HTail], NewResult):-
    Value >= Rmin,
    Value < Rmax,
    rangeFreq(Rmin, Rmax, HTail, Result),
    NewResult is Result + Counter.
         
rangeFreq(Rmin, Rmax, [[Value, _]| HTail], Result):-
    (Value > Rmax ; Value < Rmin),
	rangeFreq(Rmin, Rmax, HTail, Result).

%histogram([1,2,3], [1,1,1,1,1, 2,1,3,3,3,1,2], H).
histogram([], _List, []).
%histogram([Class_Head|Class_Tail], List, [[Class_Head, Counted] | Tail]):-
histogram([Class_Head|Class_Tail], List, [Counted | Tail]):-
	count(Class_Head,List, Counted),
	histogram(Class_Tail, List, Tail).
	
count(_Item, [], 0).
count(Item, [Item | Rest], Count) :-
    count(Item, Rest, RestCount),
    Count is RestCount + 1.
count(Item, [Other | Rest], Count) :-
    not(Other = Rest),
    count(Item, Rest, Count).

