:- discontiguous holdsAt/2, happens/2.
:- multifile holdsAt/2, happens/2.
:- dynamic holdsAt/2, happens/2.
:-['parser.prolog'].


mk(DirName, Alias, Data ):-
  length(Data, 2), 
  mkPrepare(DirName, Alias, Data, Prefix),
  mkLLE(Prefix),!.

mk(DirName, Alias, Data ):-
  length(Data, 3),
  mkPrepare(DirName, Alias, Data, Prefix),
  mkLLE(Prefix),
  mkHLE(Prefix),!.

mk(_, _, Data ):-
  length(Data, L), L < 2 ; L > 3,!.

mkPrepare(DirName, Alias, Data, Prefix):-
  openData(Data), 
  targetBasePath(TBasePath),
  mergeStrings(TBasePath, DirName, TPath),
  make_directory(TPath),
  mergeStrings('/', Alias, A),
  mergeStrings(TPath, A, Prefix).

mkLLE(Prefix):-
  %   write LLE
  mergeStrings(Prefix, 'LLE.db', FileLLE),
  extractLLE(FileLLE),
  %   write Close
  mergeStrings(Prefix, 'Close.db', FileClose),
  extractClose(FileClose),
  %   write OM
  mergeStrings(Prefix, 'OM.db', FileOM),
  extractOM(FileOM),
  %   write IDs
  mergeStrings(Prefix, '_ids.txt', FileIDs),
  extractIDs(FileIDs),
  %   write event symbols
  mergeStrings(Prefix, '_events.txt', FileEvents),
  extractEvents(FileEvents),
  %   write Sequence
  mergeStrings(Prefix, '_sequence.db', FileSeq),
  extractSeq(FileSeq).

mkHLE(Prefix ):-
  %  write HLE   
  mergeStrings(Prefix, 'ContextGrp.db', FileHLE),
  extractHLE(FileHLE).


makeFor(DirName, Alias, Flag):-
  Flag = true,
  sourceBasePath(S),
  mergeStrings(S, DirName, D1),
  mergeStrings(D1, '/', D2),
  mergeStrings(D2, Alias, Prefix),
  %   AppearenceIndv.pl
  mergeStrings(Prefix, 'AppearenceIndv.pl', F1),
  %   MovementIndv.pl
  mergeStrings(Prefix, 'MovementIndv.pl', F2),
  %   ContextGrp.pl
  mergeStrings(Prefix, 'ContextGrp.pl', F3),
  % 
  mk(DirName, Alias, [F1,F2,F3]).

  
makeFor(DirName, Alias, Flag):-
  Flag = false,
  sourceBasePath(S),
  mergeStrings(S, DirName, D1),
  mergeStrings(D1, '/', D2),
  mergeStrings(D2, Alias, Prefix),
  %   AppearenceIndv.pl
  mergeStrings(Prefix, 'AppearenceIndv.pl', F1),
  %   MovementIndv.pl
  mergeStrings(Prefix, 'MovementIndv.pl', F2),
  % 
  mk(DirName, Alias, [F1,F2]).

makeFor(_, _, Flag):- not(Flag = true ; Flag = false ).


make([H|T]):-
  make(H),
  make(T).

make([]).

make('01-Walk1'):- makeFor('01-Walk1', 'wk1gt', false).

make('02-Walk2'):- makeFor('02-Walk2', 'wk2gt', false).

make('03-Walk3'):-  makeFor('03-Walk3', 'wk3gt', false).

make('04-Browse1'):- makeFor('04-Browse1', 'br1gt', false).

make('05-Browse2'):- makeFor('05-Browse2', 'br2gt', false).

make('06-Browse3'):- makeFor('06-Browse3', 'br3gt', false).

make('07-Browse4'):- makeFor('07-Browse4', 'br4gt', false).

make('08-Browse_WhileWaiting1'):- makeFor('08-Browse_WhileWaiting1', 'bww1gt', false).

make('09-Browse_WhileWaiting2'):- makeFor('09-Browse_WhileWaiting2', 'bww2gt', false).

make('10-Rest_InChair'):- makeFor('10-Rest_InChair', 'ricgt', false).

make('11-Rest_SlumpOnFloor'):- makeFor('11-Rest_SlumpOnFloor', 'rsfgt', false).

make('12-Rest_WiggleOnFloor'):- makeFor('12-Rest_WiggleOnFloor', 'rwgt', false).

make('13-Rest_FallOnFloor'):- makeFor('13-Rest_FallOnFloor', 'rffgt', false).

make('14-LeftBag'):- makeFor('14-LeftBag', 'lb1gt', false).


make('15-LeftBag_AtChair'):- makeFor('15-LeftBag_AtChair', 'lb2gt', false).


% % % % % % % % % 
% 16,17,18 ?? 
% % % % % % % % % 
make('16-LeftBag_BehindChair'):- makeFor('16-LeftBag_BehindChair', 'lbbcgt', false).
make('17-LeftBox'):- makeFor('17-LeftBox', 'lbgt', false).
make('18-LeftBag_PickedUp'):- makeFor('18-LeftBag_PickedUp', 'lbpugt', false).
% % % % % % % % % 

make('19-Meet_WalkTogether1'):- makeFor('19-Meet_WalkTogether1', 'mwt1gt', true).

make('20-Meet_WalkTogether2'):- makeFor('20-Meet_WalkTogether2', 'mwt2gt', false).

make('21-Meet_WalkSplit'):- makeFor('21-Meet_WalkSplit', 'mws1gt', false).

make('22-Meet_Split3rdGuy'):- makeFor('22-Meet_Split3rdGuy', 'ms3ggt', false).

make('23-Meet_Crowd'):- makeFor('23-Meet_Crowd', 'mc1gt', false).

make('24-Meet_Split'):- makeFor('24-Meet_Split', 'spgt', true).

make('25-Fight_RunAway1'):- makeFor('25-Fight_RunAway1', 'fra1gt', true).

make('26-Fight_RunAway2'):- makeFor('26-Fight_RunAway2', 'fra2gt', false).

make('27-Fight_OneManDown1'):- makeFor('27-Fight_OneManDown1', 'fomdgt1', false).

make('27-Fight_OneManDown2'):- makeFor('27-Fight_OneManDown2', 'fomdgt2', false).

make('27-Fight_OneManDown3'):- makeFor('27-Fight_OneManDown3', 'fomdgt3', false).

make('28-Fight_Chase'):- makeFor('28-Fight_Chase', 'fcgt', false). 


test19to28 :-
make(['19-Meet_WalkTogether1', '20-Meet_WalkTogether2', '21-Meet_WalkSplit',
 '22-Meet_Split3rdGuy', '23-Meet_Crowd','24-Meet_Split', '25-Fight_RunAway1',
 '26-Fight_RunAway2', '27-Fight_OneManDown1', '27-Fight_OneManDown2', '27-Fight_OneManDown3', '28-Fight_Chase' ]).
