#!/bin/bash


make_functions(){
  EVENTS=$1
  FLUENTS=$2
  IDS=""
  idsfile=`ls *_ids.txt`
  if [ ! -f $idsfile ]; then
    echo "cannot find *_events.txt file."
    exit 1
  fi

  # Collect IDs
  while read current_id; do
    IDS=$IDS" "${current_id:2}
  done < $idsfile
    
  c=$((${#idsfile} - 8))
  file_prefix=${idsfile:0:$c}


  functions_mln=${file_prefix}"_functions.mln"
  functions_db=${file_prefix}"_functions.db"

  # Create/ovewrite files
  if [ -f "${functions_mln}" -o -f "${functions_db}" ]; then
    echo ""> ${functions_mln}
    echo ""> ${functions_db}
  else
    touch ${functions_mln}
    touch ${functions_db}
  fi

  # 
  # Domain
  # 
  for event in $EVENTS
  do
    echo "event "${event,,}"(id)" >>${functions_mln}
  done

  for fluent in $FLUENTS
  do
    echo "fluent "${fluent,,}"(id,id)" >>${functions_mln}
  done


  # 
  # Event functions
  # 
  echo "// Events" >>${functions_db}
  for event in $EVENTS
  do
    for i in $IDS
    do 
      echo ${event}"_ID"$i" = "${event,,}"(ID"$i")" >>${functions_db}
    done
  done

  # 
  # Fluent functions
  # 
  echo "// Fluents" >>${functions_db}
  for fluent in $FLUENTS
  do
    for i in $IDS
    do 
      for j in $IDS
      do
	echo ${fluent}"_ID"$i"_ID"$j" = "${fluent,,}"(ID"$i", ID"$j")" >>${functions_db}
      done
    done
  done
}


echo "CAVIAR"
for dir_name in `ls ./mln/caviar`
do
  echo "Building function mappings for: "$dir_name
  cd "./mln/caviar/"$dir_name 
  make_functions "Enter Exit Walking Running Active Inactive" "Meet Fight Move Leaving_object"
  cd -
done

