#!/bin/bash

#
# Settings
#
OVERRIDE_LLE_NAMES='true'
ORDERED_FUNCTION_MAPPINGS='false'
INCLUDE_FUNCTION_MAPPINGS='false'

# Low-level event names:
LLE_NAMES="Enter Exit Walking Running Active Inactive"
# High-level event names:
HLE_NAMES="Meet Fight Move Leaving_object"


capFirstLetter(){
  local new_str=""
  for i in $1; do
    new_str=$new_str`echo "${i:0:1}" | tr a-z A-Z`${i:1}
  done
  echo $new_str
}


write_evidence(){
#   EVENTS=$1
  if [ $OVERRIDE_LLE_NAMES = 'false' ]; then
    evsfile=`ls *_events.txt`
    if [ ! -f $evsfile ]; then
      echo "cannot find *_events.txt file."
      exit 1
    fi
    EVENTS=""
    while read current_ev; do
      EVENTS=$EVENTS" "$current_ev
    done < $evsfile
  else
    EVENTS=$1
  fi


  FLUENTS=$2
  IDS=""
  idsfile=`ls *_ids.txt`
  if [ ! -f $idsfile ]; then
    echo "cannot find *_ids.txt file."
    exit 1
  fi

  # Collect IDs
  while read current_id; do
    IDS=$IDS" "${current_id:2}
  done < $idsfile
    
  c=$((${#idsfile} - 8))
  file_prefix=${idsfile:0:$c}

  evidence_file=${file_prefix}"_evidence.db"

  # Create/ovewrite evidence file
  echo ""> ${evidence_file}

  if [ $INCLUDE_FUNCTION_MAPPINGS = true ]; then
     functions_file=${evidence_file}
  else
     functions_file=${file_prefix}"_functions.db"
     # Create/ovewrite functions file
     echo ""> $functions_file
  fi


  # 
  # Event function mappings
  # 
  echo "// Events" >>${functions_file}
  for event in $EVENTS
  do
    ev_constant=$(capFirstLetter $event)
    for i in $IDS
    do 
      echo ${ev_constant}"_ID"$i" = "${event,,}"(ID"$i")" >>${functions_file}
    done
  done

  # 
  # Fluent function mappings
  # 
  echo "// Fluents" >>${functions_file}
  for fluent in $FLUENTS
  do
    for i in $IDS
    do 
      for j in $IDS
      do
	if [ $ORDERED_FUNCTION_MAPPINGS = 'false' ]; then
	  echo ${fluent}"_ID"$i"_ID"$j" = "${fluent,,}"(ID"$i", ID"$j")" >>${functions_file}
	else
	  if [ $i -le $j ]; then
	    echo ${fluent}"_ID"$i"_ID"$j" = "${fluent,,}"(ID"$i", ID"$j")" >>${functions_file}
	  else
	    echo ${fluent}"_ID"$j"_ID"$i" = "${fluent,,}"(ID"$i", ID"$j")" >>${functions_file}
	  fi
	fi
      done
    done
  done

  echo "// Function mappings" >>${evidence_file}
  cat ${file_prefix}_functions.db >>${evidence_file}
  echo "// Low-level events:" >>${evidence_file}
  cat ${file_prefix}LLE.db >>${evidence_file}
  echo "// LLE Close:" >>${evidence_file}
  cat ${file_prefix}Close.db >>${evidence_file}
  echo "// LLE OrientationMove:" >>${evidence_file}
  cat ${file_prefix}OM.db >>${evidence_file}
  echo "// Sequence:" >>${evidence_file}
  cat ${file_prefix}_sequence.db >>${evidence_file}
}

mergeEvidence(){
  SRC_PATH=$1
  for dir_name in `ls $SRC_PATH`
  do
    TARGET=$SRC_PATH"/"$dir_name
    if [ -d  $TARGET ]; then
      echo "Merging evidence in "$TARGET
      cd $TARGET 
      write_evidence "${LLE_NAMES}" "${HLE_NAMES}"
      cd -
    fi
  done
}

# EXEC merging

mergeEvidence "mln/caviar"
